@@ -2,11 +2,14 @@ from __future__ import unicode_literals
 
 # Colors definitions
 colors = {
-    "OK":        "\033[92m",
-    "INFO":      "\033[93m",
-    "ERROR":     "\033[31m",
-    "TITLE":     "\033[93m",
     "TEXT":      "\033[0m",
+    "":          "\033[0m",
+    "TITLE":     "\033[93m",
+    "INFO":      "\033[93m",
+    "WARNING":   "\033[38;2;255;103;0m",
+    "ORANGE":   "\033[38;2;255;103;0m",
+    "ERROR":     "\033[31m",
+    "OK":        "\033[92m",
     "VULN":      "\033[31m",
     "NOTVULN":   "\033[92m",
     "REDIR":     "\033[0m",
@@ -17,15 +20,17 @@ colors = {
 
 # Chars definitions
 chars = {
-    "OK":        "\u2713",
-    "INFO":      "i",
-    "ERROR":     "\u2717",
-    "TITLE":     "*",
     "TEXT":      "",
+    "":          "",
+    "TITLE":     "*",
+    "INFO":      "+",
+    "WARNING":   "!",
+    "ERROR":     "\u2717",
+    "OK":        "\u2713",
     "VULN":      "\u2717",
     "NOTVULN":   "\u2713",
     "REDIR":     "\u2794",
     "PARSED":    "\u26AD",
     "TITNOBULL": "",
-    "ADDITIONS": ""
+    "ADDITIONS": "",
 }
\ No newline at end of file
